@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2017-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -95,7 +95,7 @@
  * should be very rare.  Because the L1 directory is stored in order
  * of offset, we can use an efficient binary search for lookups.
  *
- * Each L1 directory entry can address up to PAGE_SIZE*L2_SIZE bytes
+ * Each L1 directory entry can address up to SPARSE_PAGE*L2_SIZE bytes
  * in the virtual disk image.  With the current parameters this is
  * 128MB, which is enough for a 100MB image to fit into a single L1
  * directory, or a 10GB image to fit into 80 L1 entries.  The page
@@ -112,11 +112,11 @@
  *                              │ page L2_SIZE-1  ─────────▶ page
  *                              └────────────────────┘
  */
-#define PAGE_SIZE 32768
-#define L2_SIZE   4096
+#define SPARSE_PAGE 32768
+#define L2_SIZE     4096
 
 struct l2_entry {
-  void *page;                   /* Pointer to page (array of PAGE_SIZE bytes).*/
+  void *page;                /* Pointer to page (array of SPARSE_PAGE bytes).*/
 };
 
 struct l1_entry {
@@ -124,10 +124,15 @@ struct l1_entry {
   struct l2_entry *l2_dir;      /* Pointer to L2 directory (L2_SIZE entries). */
 };
 
-DEFINE_VECTOR_TYPE(l1_dir, struct l1_entry);
+DEFINE_VECTOR_TYPE (l1_dir, struct l1_entry);
 
 struct sparse_array {
   struct allocator a;           /* Must come first. */
+
+  /* This lock is highly contended.  When hammering the memory plugin
+   * with 8 fio threads, about 30% of the total system time was taken
+   * just waiting for this lock.  Fixing this is quite hard.
+   */
   pthread_mutex_t lock;
   l1_dir l1_dir;                /* L1 directory. */
 };
@@ -172,7 +177,7 @@ compare_l1_offsets (const void *offsetp, const struct l1_entry *e)
   const uint64_t offset = *(uint64_t *)offsetp;
 
   if (offset < e->offset) return -1;
-  if (offset >= e->offset + PAGE_SIZE*L2_SIZE) return 1;
+  if (offset >= e->offset + SPARSE_PAGE*L2_SIZE) return 1;
   return 0;
 }
 
@@ -236,7 +241,7 @@ lookup (struct sparse_array *sa, uint64_t offset, bool create,
   void *page;
   struct l1_entry new_entry;
 
-  *remaining = PAGE_SIZE - (offset & (PAGE_SIZE-1));
+  *remaining = SPARSE_PAGE - (offset & (SPARSE_PAGE-1));
 
  again:
   /* Search the L1 directory. */
@@ -254,13 +259,13 @@ lookup (struct sparse_array *sa, uint64_t offset, bool create,
     l2_dir = entry->l2_dir;
 
     /* Which page in the L2 directory? */
-    o = (offset - entry->offset) / PAGE_SIZE;
+    o = (offset - entry->offset) / SPARSE_PAGE;
     if (l2_entry)
       *l2_entry = &l2_dir[o];
     page = l2_dir[o].page;
     if (!page && create) {
       /* No page allocated.  Allocate one if creating. */
-      page = calloc (PAGE_SIZE, 1);
+      page = calloc (SPARSE_PAGE, 1);
       if (page == NULL) {
         nbdkit_error ("calloc: %m");
         return NULL;
@@ -270,7 +275,7 @@ lookup (struct sparse_array *sa, uint64_t offset, bool create,
     if (!page)
       return NULL;
     else
-      return page + (offset & (PAGE_SIZE-1));
+      return page + (offset & (SPARSE_PAGE-1));
   }
 
   /* No L1 directory entry found. */
@@ -282,7 +287,7 @@ lookup (struct sparse_array *sa, uint64_t offset, bool create,
    * allocate the L2 directory with NULL page pointers.  Then we can
    * repeat the above search to create the page.
    */
-  new_entry.offset = offset & ~(PAGE_SIZE*L2_SIZE-1);
+  new_entry.offset = offset & ~(SPARSE_PAGE*L2_SIZE-1);
   new_entry.l2_dir = calloc (L2_SIZE, sizeof (struct l2_entry));
   if (new_entry.l2_dir == NULL) {
     nbdkit_error ("calloc: %m");
@@ -395,13 +400,13 @@ sparse_array_zero (struct allocator *a, uint64_t count, uint64_t offset)
       n = count;
 
     if (p) {
-      if (n < PAGE_SIZE)
+      if (n < SPARSE_PAGE)
         memset (p, 0, n);
       else
         assert (p == l2_entry->page);
 
       /* If the whole page is now zero, free it. */
-      if (n >= PAGE_SIZE || is_zero (l2_entry->page, PAGE_SIZE)) {
+      if (n >= SPARSE_PAGE || is_zero (l2_entry->page, SPARSE_PAGE)) {
         if (sa->a.debug)
           nbdkit_debug ("%s: freeing zero page at offset %" PRIu64,
                         __func__, offset);
@@ -447,7 +452,7 @@ sparse_array_blit (struct allocator *a1,
       return -1;
 
     /* If the whole page is now zero, free it. */
-    if (is_zero (l2_entry->page, PAGE_SIZE)) {
+    if (is_zero (l2_entry->page, SPARSE_PAGE)) {
       if (sa2->a.debug)
         nbdkit_debug ("%s: freeing zero page at offset %" PRIu64,
                       __func__, offset2);
@@ -536,7 +541,7 @@ static struct allocator_functions functions = {
   .extents = sparse_array_extents,
 };
 
-static void register_sparse_array (void) __attribute__((constructor));
+static void register_sparse_array (void) __attribute__ ((constructor));
 
 static void
 register_sparse_array (void)
