@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -40,8 +40,8 @@
  * these structures.
  */
 
-#if defined(__GNUC__) || defined(__clang__)
-#define NBD_ATTRIBUTE_PACKED __attribute__((__packed__))
+#if defined (__GNUC__) || defined (__clang__)
+#define NBD_ATTRIBUTE_PACKED __attribute__ ((__packed__))
 #else
 #error "Please port to your compiler's notion of a packed struct"
 #endif
@@ -58,8 +58,8 @@ struct nbd_old_handshake {
   char zeroes[124];           /* must be sent as zero bytes */
 } NBD_ATTRIBUTE_PACKED;
 
-#define NBD_MAGIC       UINT64_C(0x4e42444d41474943) /* ASCII "NBDMAGIC" */
-#define NBD_OLD_VERSION UINT64_C(0x0000420281861253)
+#define NBD_MAGIC       UINT64_C (0x4e42444d41474943) /* ASCII "NBDMAGIC" */
+#define NBD_OLD_VERSION UINT64_C (0x0000420281861253)
 
 /* New-style handshake. */
 struct nbd_new_handshake {
@@ -68,7 +68,7 @@ struct nbd_new_handshake {
   uint16_t gflags;            /* global flags */
 } NBD_ATTRIBUTE_PACKED;
 
-#define NBD_NEW_VERSION UINT64_C(0x49484156454F5054) /* ASCII "IHAVEOPT" */
+#define NBD_NEW_VERSION UINT64_C (0x49484156454F5054) /* ASCII "IHAVEOPT" */
 
 /* New-style handshake option (sent by the client to us). */
 struct nbd_new_option {
@@ -95,7 +95,7 @@ struct nbd_fixed_new_option_reply {
   uint32_t replylen;
 } NBD_ATTRIBUTE_PACKED;
 
-#define NBD_REP_MAGIC UINT64_C(0x3e889045565a9)
+#define NBD_REP_MAGIC UINT64_C (0x3e889045565a9)
 
 /* Global flags. */
 #define NBD_FLAG_FIXED_NEWSTYLE    (1 << 0)
@@ -154,6 +154,14 @@ struct nbd_fixed_new_option_reply_info_export {
   uint16_t eflags;              /* per-export flags */
 } NBD_ATTRIBUTE_PACKED;
 
+/* NBD_INFO_NAME or NBD_INFO_DESCRIPTION reply (follows
+ * fixed_new_option_reply).
+ */
+struct nbd_fixed_new_option_reply_info_name_or_desc {
+  uint16_t info;                /* NBD_INFO_NAME, NBD_INFO_DESCRIPTION */
+  /* followed by a string name or description */
+} NBD_ATTRIBUTE_PACKED;
+
 /* NBD_INFO_BLOCK_SIZE reply (follows fixed_new_option_reply). */
 struct nbd_fixed_new_option_reply_info_block_size {
   uint16_t info;                /* NBD_INFO_BLOCK_SIZE */
@@ -165,7 +173,7 @@ struct nbd_fixed_new_option_reply_info_block_size {
 /* NBD_REP_SERVER reply (follows fixed_new_option_reply). */
 struct nbd_fixed_new_option_reply_server {
   uint32_t export_name_len;     /* length of export name */
-  /* followed by a string export name and description*/
+  /* followed by a string export name and description */
 } NBD_ATTRIBUTE_PACKED;
 
 /* NBD_REP_META_CONTEXT reply (follows fixed_new_option_reply). */
@@ -185,7 +193,7 @@ struct nbd_request {
   uint32_t magic;               /* NBD_REQUEST_MAGIC. */
   uint16_t flags;               /* Request flags. */
   uint16_t type;                /* Request type. */
-  uint64_t handle;              /* Opaque handle. */
+  uint64_t cookie;              /* Opaque handle. */
   uint64_t offset;              /* Request offset. */
   uint32_t count;               /* Request length. */
 } NBD_ATTRIBUTE_PACKED;
@@ -194,7 +202,7 @@ struct nbd_request {
 struct nbd_simple_reply {
   uint32_t magic;               /* NBD_SIMPLE_REPLY_MAGIC. */
   uint32_t error;               /* NBD_SUCCESS or one of NBD_E*. */
-  uint64_t handle;              /* Opaque handle. */
+  uint64_t cookie;              /* Opaque handle. */
 } NBD_ATTRIBUTE_PACKED;
 
 /* Structured reply (server -> client). */
@@ -202,7 +210,7 @@ struct nbd_structured_reply {
   uint32_t magic;               /* NBD_STRUCTURED_REPLY_MAGIC. */
   uint16_t flags;               /* NBD_REPLY_FLAG_* */
   uint16_t type;                /* NBD_REPLY_TYPE_* */
-  uint64_t handle;              /* Opaque handle. */
+  uint64_t cookie;              /* Opaque handle. */
   uint32_t length;              /* Length of payload which follows. */
 } NBD_ATTRIBUTE_PACKED;
 
