@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -72,7 +72,8 @@ partition_config_complete (nbdkit_next_config_complete *next,
                            nbdkit_backend *nxdata)
 {
   if (partnum == 0) {
-    nbdkit_error ("you must supply the partition parameter on the command line");
+    nbdkit_error ("you must supply the partition parameter on the command "
+                  "line");
     return -1;
   }
 
@@ -305,4 +306,4 @@ static struct nbdkit_filter filter = {
   .cache              = partition_cache,
 };
 
-NBDKIT_REGISTER_FILTER(filter)
+NBDKIT_REGISTER_FILTER (filter)
