@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2018-2021 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -96,8 +96,9 @@ tar_config_complete (nbdkit_next_config_complete *next,
   return next (nxdata);
 }
 
-#define tar_config_help \
-  "tar-entry=<FILENAME> (required) The path inside the tar file to serve."
+#define tar_config_help                                                 \
+  "tar-entry=<FILENAME> (required) The path inside the tar file to serve.\n" \
+  "tar=<PATH>                      Path of the tar binary."
 
 static int
 tar_thread_model (void)
@@ -429,4 +430,4 @@ static struct nbdkit_filter filter = {
   .cache              = tar_cache,
 };
 
-NBDKIT_REGISTER_FILTER(filter)
+NBDKIT_REGISTER_FILTER (filter)
