@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -50,64 +50,75 @@ extern "C" {
 #define NBDKIT_ZERO_NATIVE   2
 
 #ifdef NBDKIT_INTERNAL
-/* Opaque type encapsulating all information needed for calling into
+/* Opaque types encapsulating all information needed for calling into
  * the next filter or plugin.
  */
 typedef struct backend nbdkit_backend;
+typedef struct context nbdkit_context;
+typedef struct context nbdkit_next;
 #else
-typedef void nbdkit_backend;
+typedef struct nbdkit_backend nbdkit_backend;
+typedef struct nbdkit_context nbdkit_context;
+typedef struct nbdkit_next_ops nbdkit_next;
 #endif
 
 /* Next ops. */
 typedef int nbdkit_next_config (nbdkit_backend *nxdata,
                                 const char *key, const char *value);
 typedef int nbdkit_next_config_complete (nbdkit_backend *nxdata);
-typedef int nbdkit_next_get_ready (nbdkit_backend *nxdata);
-typedef int nbdkit_next_after_fork (nbdkit_backend *nxdata);
 typedef int nbdkit_next_preconnect (nbdkit_backend *nxdata, int readonly);
 typedef int nbdkit_next_list_exports (nbdkit_backend *nxdata, int readonly,
                                       struct nbdkit_exports *exports);
 typedef const char *nbdkit_next_default_export (nbdkit_backend *nxdata,
                                                 int readonly);
-typedef int nbdkit_next_open (nbdkit_backend *nxdata,
+typedef int nbdkit_next_open (nbdkit_context *context,
                               int readonly, const char *exportname);
 
 struct nbdkit_next_ops {
-  /* Performs close + open on the underlying chain.
-   * Used by the retry filter.
+  /* These callbacks are only needed when managing the backend manually
+   * rather than via nbdkit_next_open.
    */
-  int (*reopen) (nbdkit_backend *nxdata, int readonly, const char *exportname);
+  int (*prepare) (nbdkit_next *nxdata);
+  int (*finalize) (nbdkit_next *nxdata);
 
-  /* The rest of the next ops are the same as normal plugin operations. */
-  int64_t (*get_size) (nbdkit_backend *nxdata);
-  const char * (*export_description) (nbdkit_backend *nxdata);
+  /* These callbacks are the same as normal plugin operations. */
+  int64_t (*get_size) (nbdkit_next *nxdata);
+  const char * (*export_description) (nbdkit_next *nxdata);
+  int (*block_size) (nbdkit_next *nxdata,
+                     uint32_t *minimum, uint32_t *preferred, uint32_t *maximum);
 
-  int (*can_write) (nbdkit_backend *nxdata);
-  int (*can_flush) (nbdkit_backend *nxdata);
-  int (*is_rotational) (nbdkit_backend *nxdata);
-  int (*can_trim) (nbdkit_backend *nxdata);
-  int (*can_zero) (nbdkit_backend *nxdata);
-  int (*can_fast_zero) (nbdkit_backend *nxdata);
-  int (*can_extents) (nbdkit_backend *nxdata);
-  int (*can_fua) (nbdkit_backend *nxdata);
-  int (*can_multi_conn) (nbdkit_backend *nxdata);
-  int (*can_cache) (nbdkit_backend *nxdata);
+  int (*can_write) (nbdkit_next *nxdata);
+  int (*can_flush) (nbdkit_next *nxdata);
+  int (*is_rotational) (nbdkit_next *nxdata);
+  int (*can_trim) (nbdkit_next *nxdata);
+  int (*can_zero) (nbdkit_next *nxdata);
+  int (*can_fast_zero) (nbdkit_next *nxdata);
+  int (*can_extents) (nbdkit_next *nxdata);
+  int (*can_fua) (nbdkit_next *nxdata);
+  int (*can_multi_conn) (nbdkit_next *nxdata);
+  int (*can_cache) (nbdkit_next *nxdata);
 
-  int (*pread) (nbdkit_backend *nxdata,
+  int (*pread) (nbdkit_next *nxdata,
                 void *buf, uint32_t count, uint64_t offset,
                 uint32_t flags, int *err);
-  int (*pwrite) (nbdkit_backend *nxdata,
+  int (*pwrite) (nbdkit_next *nxdata,
                  const void *buf, uint32_t count, uint64_t offset,
                  uint32_t flags, int *err);
-  int (*flush) (nbdkit_backend *nxdata, uint32_t flags, int *err);
-  int (*trim) (nbdkit_backend *nxdata, uint32_t count, uint64_t offset,
+  int (*flush) (nbdkit_next *nxdata, uint32_t flags, int *err);
+  int (*trim) (nbdkit_next *nxdata, uint32_t count, uint64_t offset,
                uint32_t flags, int *err);
-  int (*zero) (nbdkit_backend *nxdata, uint32_t count, uint64_t offset,
+  int (*zero) (nbdkit_next *nxdata, uint32_t count, uint64_t offset,
                uint32_t flags, int *err);
-  int (*extents) (nbdkit_backend *nxdata, uint32_t count, uint64_t offset,
+  int (*extents) (nbdkit_next *nxdata, uint32_t count, uint64_t offset,
                   uint32_t flags, struct nbdkit_extents *extents, int *err);
-  int (*cache) (nbdkit_backend *nxdata, uint32_t count, uint64_t offset,
+  int (*cache) (nbdkit_next *nxdata, uint32_t count, uint64_t offset,
                 uint32_t flags, int *err);
+
+  /* Note: Actual instances of this struct contain additional opaque
+   * data not listed in this header; you cannot manually copy or
+   * initialize sizeof(struct nbdkit_next_ops) bytes, but must instead
+   * use unchanged pointers obtained from the nbdkit API.
+   */
 };
 
 /* Extent functions. */
@@ -117,16 +128,19 @@ struct nbdkit_extent {
   uint32_t type;
 };
 
-NBDKIT_EXTERN_DECL (struct nbdkit_extents *,nbdkit_extents_new,
+NBDKIT_EXTERN_DECL (struct nbdkit_extents *, nbdkit_extents_new,
                     (uint64_t start, uint64_t end));
 NBDKIT_EXTERN_DECL (void, nbdkit_extents_free, (struct nbdkit_extents *));
 NBDKIT_EXTERN_DECL (size_t, nbdkit_extents_count,
                     (const struct nbdkit_extents *));
 NBDKIT_EXTERN_DECL (struct nbdkit_extent, nbdkit_get_extent,
                     (const struct nbdkit_extents *, size_t));
+NBDKIT_EXTERN_DECL (struct nbdkit_extents *, nbdkit_extents_full,
+                    (nbdkit_next *next,
+                     uint32_t count, uint64_t offset,
+                     uint32_t flags, int *err));
 NBDKIT_EXTERN_DECL (int, nbdkit_extents_aligned,
-                    (struct nbdkit_next_ops *next_ops,
-                     nbdkit_backend *nxdata,
+                    (nbdkit_next *next,
                      uint32_t count, uint64_t offset,
                      uint32_t flags, uint32_t align,
                      struct nbdkit_extents *extents, int *err));
@@ -145,6 +159,16 @@ NBDKIT_EXTERN_DECL (size_t, nbdkit_exports_count,
 NBDKIT_EXTERN_DECL (const struct nbdkit_export, nbdkit_get_export,
                     (const struct nbdkit_exports *, size_t));
 
+/* Manual management of backend access. */
+NBDKIT_EXTERN_DECL (nbdkit_backend *, nbdkit_context_get_backend,
+                    (nbdkit_context *context));
+NBDKIT_EXTERN_DECL (nbdkit_next *, nbdkit_next_context_open,
+                    (nbdkit_backend *backend, int readonly,
+                     const char *exportname, int shared));
+NBDKIT_EXTERN_DECL (void, nbdkit_next_context_close, (nbdkit_next *next));
+NBDKIT_EXTERN_DECL (nbdkit_next *, nbdkit_context_set_next,
+                    (nbdkit_context *context, nbdkit_next *next));
+
 /* Filter struct. */
 struct nbdkit_filter {
   /* Do not set these fields directly; use NBDKIT_REGISTER_FILTER.
@@ -173,9 +197,9 @@ struct nbdkit_filter {
                           nbdkit_backend *nxdata);
   const char *config_help;
   int (*thread_model) (void);
-  int (*get_ready) (nbdkit_next_get_ready *next, nbdkit_backend *nxdata,
-                    int thread_model);
-  int (*after_fork) (nbdkit_next_after_fork *next, nbdkit_backend *nxdata);
+  int (*get_ready) (int thread_model);
+  int (*after_fork) (nbdkit_backend *backend);
+  void (*cleanup) (nbdkit_backend *backend);
   int (*preconnect) (nbdkit_next_preconnect *next, nbdkit_backend *nxdata,
                      int readonly);
   int (*list_exports) (nbdkit_next_list_exports *next, nbdkit_backend *nxdata,
@@ -185,66 +209,65 @@ struct nbdkit_filter {
                                   nbdkit_backend *nxdata,
                                   int readonly, int is_tls);
 
-  void * (*open) (nbdkit_next_open *next, nbdkit_backend *nxdata,
+  void * (*open) (nbdkit_next_open *next, nbdkit_context *context,
                   int readonly, const char *exportname, int is_tls);
   void (*close) (void *handle);
 
-  int (*prepare) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*prepare) (nbdkit_next *next,
                   void *handle, int readonly);
-  int (*finalize) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*finalize) (nbdkit_next *next,
                    void *handle);
 
-  int64_t (*get_size) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int64_t (*get_size) (nbdkit_next *next,
                        void *handle);
-  const char * (*export_description) (struct nbdkit_next_ops *next_ops,
-                                      nbdkit_backend *nxdata, void *handle);
+  const char * (*export_description) (nbdkit_next *next, void *handle);
+  int (*block_size) (nbdkit_next *next, void *handle,
+                     uint32_t *minimum, uint32_t *preferred, uint32_t *maximum);
 
-  int (*can_write) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*can_write) (nbdkit_next *next,
                     void *handle);
-  int (*can_flush) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*can_flush) (nbdkit_next *next,
                     void *handle);
-  int (*is_rotational) (struct nbdkit_next_ops *next_ops,
-                        nbdkit_backend *nxdata, void *handle);
-  int (*can_trim) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*is_rotational) (nbdkit_next *next, void *handle);
+  int (*can_trim) (nbdkit_next *next,
                    void *handle);
-  int (*can_zero) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*can_zero) (nbdkit_next *next,
                    void *handle);
-  int (*can_fast_zero) (struct nbdkit_next_ops *next_ops,
-                        nbdkit_backend *nxdata, void *handle);
-  int (*can_extents) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*can_fast_zero) (nbdkit_next *next, void *handle);
+  int (*can_extents) (nbdkit_next *next,
                       void *handle);
-  int (*can_fua) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*can_fua) (nbdkit_next *next,
                   void *handle);
-  int (*can_multi_conn) (struct nbdkit_next_ops *next_ops,
-                         nbdkit_backend *nxdata, void *handle);
-  int (*can_cache) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*can_multi_conn) (nbdkit_next *next, void *handle);
+  int (*can_cache) (nbdkit_next *next,
                     void *handle);
 
-  int (*pread) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*pread) (nbdkit_next *next,
                 void *handle, void *buf, uint32_t count, uint64_t offset,
                 uint32_t flags, int *err);
-  int (*pwrite) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*pwrite) (nbdkit_next *next,
                  void *handle,
                  const void *buf, uint32_t count, uint64_t offset,
                  uint32_t flags, int *err);
-  int (*flush) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*flush) (nbdkit_next *next,
                 void *handle, uint32_t flags, int *err);
-  int (*trim) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*trim) (nbdkit_next *next,
                void *handle, uint32_t count, uint64_t offset, uint32_t flags,
                int *err);
-  int (*zero) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*zero) (nbdkit_next *next,
                void *handle, uint32_t count, uint64_t offset, uint32_t flags,
                int *err);
-  int (*extents) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*extents) (nbdkit_next *next,
                   void *handle, uint32_t count, uint64_t offset, uint32_t flags,
                   struct nbdkit_extents *extents, int *err);
-  int (*cache) (struct nbdkit_next_ops *next_ops, nbdkit_backend *nxdata,
+  int (*cache) (nbdkit_next *next,
                 void *handle, uint32_t count, uint64_t offset, uint32_t flags,
                 int *err);
 };
 
 #define NBDKIT_REGISTER_FILTER(filter)                                  \
   NBDKIT_CXX_LANG_C                                                     \
+  NBDKIT_DLL_PUBLIC                                                     \
   struct nbdkit_filter *                                                \
   filter_init (void)                                                    \
   {                                                                     \
