@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2014-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -33,24 +33,47 @@
 #ifndef NBDKIT_CURLDEFS_H
 #define NBDKIT_CURLDEFS_H
 
+#include <stdbool.h>
+
 #include "windows-compat.h"
 
+/* Macro CURL_AT_LEAST_VERSION was added in 2015 (Curl 7.43) so if the
+ * macro isn't present then Curl is very old.
+ */
+#ifdef CURL_AT_LEAST_VERSION
+#if CURL_AT_LEAST_VERSION (7, 55, 0)
+#define HAVE_CURLINFO_CONTENT_LENGTH_DOWNLOAD_T
+#endif
+#endif
+
 extern const char *url;
 
 extern const char *cainfo;
 extern const char *capath;
+extern unsigned connections;
 extern char *cookie;
+extern const char *cookiefile;
+extern const char *cookiejar;
 extern const char *cookie_script;
 extern unsigned cookie_script_renew;
+extern bool followlocation;
 extern struct curl_slist *headers;
 extern const char *header_script;
 extern unsigned header_script_renew;
+extern long http_version;
 extern char *password;
+#ifndef HAVE_CURLOPT_PROTOCOLS_STR
 extern long protocols;
+#else
+extern const char *protocols;
+#endif
 extern const char *proxy;
 extern char *proxy_password;
 extern const char *proxy_user;
 extern bool sslverify;
+extern const char *ssl_cipher_list;
+extern long ssl_version;
+extern const char *tls13_ciphers;
 extern bool tcp_keepalive;
 extern bool tcp_nodelay;
 extern uint32_t timeout;
@@ -58,21 +81,58 @@ extern const char *unix_socket_path;
 extern const char *user;
 extern const char *user_agent;
 
+extern int curl_debug_verbose;
+
 /* The per-connection handle. */
+struct handle {
+  int readonly;
+};
+
+/* The libcurl handle and some associated fields and buffers. */
 struct curl_handle {
+  /* The underlying curl handle. */
   CURL *c;
+
+  /* Index of this handle in the pool (for debugging). */
+  size_t i;
+
+  /* True if the handle is in use by a thread. */
+  bool in_use;
+
+  /* These fields are used/initialized when we create the handle. */
   bool accept_range;
   int64_t exportsize;
+
   char errbuf[CURL_ERROR_SIZE];
+
+  /* Before doing a read or write operation, set these to point to the
+   * buffer where you want the data to be stored / come from.  Note
+   * the confusing terminology from libcurl: write_* is used when
+   * reading, read_* is used when writing.
+   */
   char *write_buf;
   uint32_t write_count;
   const char *read_buf;
   uint32_t read_count;
+
+  /* Used by scripts.c */
   struct curl_slist *headers_copy;
 };
 
+/* pool.c */
+extern struct curl_handle *get_handle (void);
+extern void put_handle (struct curl_handle *ch);
+extern void free_all_handles (void);
+
 /* scripts.c */
-extern int do_scripts (struct curl_handle *h);
+extern int do_scripts (struct curl_handle *ch);
 extern void scripts_unload (void);
 
+/* Translate CURLcode to nbdkit_error. */
+#define display_curl_error(ch, r, fs, ...)                      \
+  do {                                                          \
+    nbdkit_error ((fs ": %s: %s"), ## __VA_ARGS__,              \
+                  curl_easy_strerror ((r)), (ch)->errbuf);      \
+  } while (0)
+
 #endif /* NBDKIT_CURLDEFS_H */
