@@ -1,6 +1,6 @@
 (* hey emacs, this is OCaml code: -*- tuareg -*- *)
 (* nbdkit OCaml interface
- * Copyright (C) 2014-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -89,6 +89,7 @@ let register_plugin ~name
                     ~open_connection
                     ?close
                     ~get_size
+                    ?export_description
                     ?block_size
                     ?can_cache
                     ?can_extents
@@ -110,7 +111,6 @@ let register_plugin ~name
                     ?dump_plugin
                     ?list_exports
                     ?default_export
-                    ?export_description
                     () =
   (* Set fields in the C plugin struct. *)
   set_string_field "name" name;
@@ -166,9 +166,11 @@ external realpath : string -> string = "ocaml_nbdkit_realpath"
 external nanosleep : int -> int -> unit = "ocaml_nbdkit_nanosleep"
 external export_name : unit -> string = "ocaml_nbdkit_export_name"
 external shutdown : unit -> unit = "ocaml_nbdkit_shutdown" [@@noalloc]
+external disconnect : bool -> unit = "ocaml_nbdkit_disconnect" [@@noalloc]
 external _debug : string -> unit = "ocaml_nbdkit_debug" [@@noalloc]
 let debug fs = ksprintf _debug fs
 external version : unit -> string = "ocaml_nbdkit_version"
+external api_version : unit -> int = "ocaml_nbdkit_api_version"
 external peer_pid : unit -> int64 = "ocaml_nbdkit_peer_pid"
 external peer_uid : unit -> int64 = "ocaml_nbdkit_peer_uid"
 external peer_gid : unit -> int64 = "ocaml_nbdkit_peer_gid"
