@@ -1,6 +1,6 @@
 (* hey emacs, this is OCaml code: -*- tuareg -*- *)
 (* nbdkit OCaml interface
- * Copyright (C) 2014-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -33,7 +33,8 @@
 
 (** Interface between plugins written in OCaml and the nbdkit server.
 
-    Read [nbdkit-ocaml-plugin(3)] first. *)
+    Read these man pages for additional information:
+    nbdkit-ocaml-plugin(3) and nbdkit-plugin(3) *)
 
 (** Flags passed from the server to various callbacks. *)
 type flags = flag list
@@ -96,6 +97,7 @@ val register_plugin :
 
   (* NBD negotiation. *)
   get_size: ('a -> int64) ->
+  ?export_description: ('a -> string) ->
   ?block_size: ('a -> int * int * int64) ->
   ?can_cache: ('a -> cache_flag) ->
   ?can_extents: ('a -> bool) ->
@@ -121,7 +123,6 @@ val register_plugin :
   ?dump_plugin: (unit -> unit) ->
   ?list_exports: (bool -> bool -> export list) ->
   ?default_export: (bool -> bool -> string) ->
-  ?export_description: ('a -> string) ->
 
   unit ->
   unit
@@ -156,22 +157,33 @@ val read_password : string -> string
 (* OCaml's [Filename] module can handle [absolute_path]. *)
 val realpath : string -> string
 
-(** Binding for [nbdkit_nanosleep].  Sleeps for seconds and nanoseconds. *)
+(** Binding for [nbdkit_nanosleep].
+    Sleeps for seconds and nanoseconds. *)
 val nanosleep : int -> int -> unit
 
-(** Binding for [nbdkit_export_name].  Returns the name of the
-    export as requested by the client. *)
+(** Binding for [nbdkit_export_name].
+    Returns the name of the export as requested by the client. *)
 val export_name : unit -> string
 
-(** Binding for [nbdkit_shutdown].  Requests the server shut down. *)
+(** Binding for [nbdkit_shutdown].
+    Requests the server shut down. *)
 val shutdown : unit -> unit
 
+(** Binding for [nbdkit_disconnect].
+    Requests disconnecting current client. *)
+val disconnect : bool -> unit
+
 (** Print a debug message when nbdkit is in verbose mode. *)
 val debug : ('a, unit, string, unit) format4 -> 'a
 
 (** Return the version of nbdkit that the plugin was compiled with. *)
 val version : unit -> string
 
+(** Return the nbdkit API version used by the plugin.
+
+    See [NBDKIT_API_VERSION] in nbdkit-plugin(3). *)
+val api_version : unit -> int
+
 (** Binding for [nbdkit_peer_pid]. *)
 val peer_pid : unit -> int64
 
