@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -93,6 +93,18 @@ do_shutdown (PyObject *self, PyObject *args)
   Py_RETURN_NONE;
 }
 
+/* nbdkit.disconnect */
+static PyObject *
+do_disconnect (PyObject *self, PyObject *args)
+{
+  int force;
+
+  if (!PyArg_ParseTuple (args, "p:disconnect", &force))
+    return NULL;
+  nbdkit_disconnect (force);
+  Py_RETURN_NONE;
+}
+
 /* nbdkit.parse_size */
 static PyObject *
 parse_size (PyObject *self, PyObject *args)
@@ -101,13 +113,13 @@ parse_size (PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple (args, "s:parse_size", &s))
     return NULL;
 
-  int64_t size = nbdkit_parse_size(s);
+  int64_t size = nbdkit_parse_size (s);
   if (size == -1) {
     PyErr_SetString (PyExc_ValueError, "Unable to parse string as size");
     return NULL;
   }
 
-  return PyLong_FromSize_t((size_t)size);
+  return PyLong_FromSize_t ((size_t)size);
 }
 
 static PyMethodDef NbdkitMethods[] = {
@@ -121,6 +133,8 @@ static PyMethodDef NbdkitMethods[] = {
     "Store an errno value prior to throwing an exception" },
   { "shutdown", do_shutdown, METH_NOARGS,
     "Request asynchronous shutdown" },
+  { "disconnect", do_disconnect, METH_VARARGS,
+    "Request disconnection from current client" },
   { NULL }
 };
 
