@@ -1046,6 +1046,7 @@ extern "C" {
     fn nbdkit_peer_name( addr: *mut libc::sockaddr,
                          addrlen: *mut libc::socklen_t) -> c_int;
     fn nbdkit_shutdown();
+    fn nbdkit_disconnect(force: bool);
     fn nbdkit_stdio_safe() -> c_int;
 }
 
@@ -1106,6 +1107,11 @@ pub fn shutdown() {
     unsafe { nbdkit_shutdown() };
 }
 
+/// Request nbdkit to disconnect the current client.
+pub fn disconnect(force: bool) {
+    unsafe { nbdkit_disconnect(force) };
+}
+
 #[doc(hidden)]
 #[repr(C)]
 pub struct Plugin {
