@@ -105,7 +105,7 @@ fn with_fixture<F: FnMut(&mut Fixture)>(mut f: F) {
     initialize();
     let _m = MOCK_SERVER_MTX.lock().unwrap();
 
-    let mut mock = Box::new(MockServer::default());
+    let mut mock = Box::<MockServer>::default();
     mock.expect_get_size()
         .returning(|| Ok(0x4200));
     let mockp = (&mut mock) as *mut Box<MockServer>;
@@ -115,7 +115,7 @@ fn with_fixture<F: FnMut(&mut Fixture)>(mut f: F) {
 
     let pluginp = unsafe { PLUGIN.unwrap()};
     let plugin = unsafe {&*pluginp};
-    let handle = ((*plugin).open)(0);
+    let handle = (plugin.open)(0);
     open_ctx.checkpoint();              // clear expectations for MockServer::open
     let mut fixture = Fixture {
         mockp,
@@ -125,7 +125,7 @@ fn with_fixture<F: FnMut(&mut Fixture)>(mut f: F) {
 
     f(&mut fixture);
 
-    ((*plugin).close)(handle);
+    (plugin.close)(handle);
 }
 
 /// Helper for testing methods that take a handle and return a boolean
