@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -36,22 +36,27 @@
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdarg.h>
-#include <sys/socket.h>
 #include <pthread.h>
 
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
 #define NBDKIT_API_VERSION 2
 #define NBDKIT_INTERNAL
 #include "nbdkit-plugin.h"
 #include "nbdkit-filter.h"
 #include "cleanup.h"
 #include "nbd-protocol.h"
+#include "string-vector.h"
 #include "unix-path-max.h"
 #include "vector.h"
+#include "windows-compat.h"
 
 /* Define unlikely macro, but only for GCC.  These are used to move
  * debug and error handling code out of hot paths.
  */
-#if defined(__GNUC__)
+#if defined (__GNUC__)
 #define unlikely(x) __builtin_expect (!!(x), 0)
 #define if_verbose if (unlikely (verbose))
 #else
@@ -59,11 +64,7 @@
 #define if_verbose if (verbose)
 #endif
 
-#if HAVE_VALGRIND
-# include <valgrind.h>
-/* http://valgrind.org/docs/manual/faq.html#faq.unhelpful */
-# define DO_DLCLOSE !RUNNING_ON_VALGRIND
-#elif defined(__SANITIZE_ADDRESS__)
+#if defined (__SANITIZE_ADDRESS__)
 # define DO_DLCLOSE 0
 #elif ENABLE_LIBFUZZER
 /* XXX This causes dlopen in the server to leak during fuzzing.
@@ -72,7 +73,13 @@
  */
 # define DO_DLCLOSE 0
 #else
-# define DO_DLCLOSE 1
+# if HAVE_VALGRIND
+#  include <valgrind.h>
+/* http://valgrind.org/docs/manual/faq.html#faq.unhelpful */
+#  define DO_DLCLOSE !RUNNING_ON_VALGRIND
+# else
+#  define DO_DLCLOSE 1
+# endif
 #endif
 
 /* Declare program_name. */
@@ -85,15 +92,9 @@
 
 #define container_of(ptr, type, member) ({                       \
       const typeof (((type *) 0)->member) *__mptr = (ptr);       \
-      (type *) ((char *) __mptr - offsetof(type, member));       \
+      (type *) ((char *) __mptr - offsetof (type, member));      \
     })
 
-#define debug(fs, ...)                                   \
-  do {                                                   \
-    if_verbose                                           \
-      nbdkit_debug ((fs), ##__VA_ARGS__);                \
-  } while (0)
-
 /* Maximum read or write request that we will handle. */
 #define MAX_REQUEST_SIZE (64 * 1024 * 1024)
 
@@ -107,6 +108,7 @@ enum log_to {
   LOG_TO_NULL,           /* --log=null forced on the command line */
 };
 
+extern int tcpip_sock_af;
 extern struct debug_flag *debug_flags;
 extern const char *export_name;
 extern bool foreground;
@@ -119,7 +121,6 @@ extern const char *port;
 extern bool read_only;
 extern const char *run;
 extern bool listen_stdin;
-extern bool configured;
 extern const char *selinux_label;
 extern unsigned threads;
 extern int tls;
@@ -129,6 +130,8 @@ extern bool tls_verify_peer;
 extern char *unixsocket;
 extern const char *user, *group;
 extern bool verbose;
+extern bool vsock;
+extern bool configured;
 extern int saved_stdin;
 extern int saved_stdout;
 
@@ -145,9 +148,23 @@ extern int saved_stdout;
 extern struct backend *top;
 #define for_each_backend(b) for (b = top; b != NULL; b = b->next)
 
+/* debug.c */
+#define debug(fs, ...)                                   \
+  do {                                                   \
+    if_verbose                                           \
+      debug_in_server ((fs), ##__VA_ARGS__);             \
+  } while (0)
+
+extern void debug_in_server (const char *msg, ...)
+  ATTRIBUTE_FORMAT_PRINTF (1, 2);
+
 /* quit.c */
 extern volatile int quit;
+#ifndef WIN32
 extern int quit_fd;
+#else
+extern HANDLE quit_fd;
+#endif
 extern void set_up_quit_pipe (void);
 extern void close_quit_pipe (void);
 extern void handle_quit (int sig);
@@ -177,19 +194,15 @@ enum {
 };
 
 typedef int (*connection_recv_function) (void *buf, size_t len)
-  __attribute__((__nonnull__ (1)));
+  __attribute__ ((__nonnull__ (1)));
 typedef int (*connection_send_function) (const void *buf, size_t len,
                                          int flags)
-  __attribute__((__nonnull__ (1)));
-typedef void (*connection_close_function) (void);
+  __attribute__ ((__nonnull__ (1)));
+typedef void (*connection_close_function) (int how);
 
-/* struct handle stores data per connection and backend.  Primarily
+/* struct context stores data per connection and backend.  Primarily
  * this is the filter or plugin handle, but other state is also stored
  * here.
- *
- * Use get_handle (conn, 0) to return the struct handle for the
- * plugin, and get_handle (conn, b->i) to return the struct handle for
- * the i'th backend (if b->i >= 1 then for a filter).
  */
 enum {
   HANDLE_OPEN = 1,      /* Set if .open passed, so .close is needed */
@@ -197,14 +210,20 @@ enum {
   HANDLE_FAILED = 4,    /* Set if .finalize failed */
 };
 
-struct handle {
+struct context {
+  struct nbdkit_next_ops next;  /* Must be first member, for ABI reasons */
+
   void *handle;         /* Plugin or filter handle. */
+  struct backend *b;    /* Backend that provided handle. */
+  struct context *c_next; /* Underlying context, only when b->next != NULL. */
+  struct connection *conn; /* Active connection at context creation, if any. */
 
   unsigned char state;  /* Bitmask of HANDLE_* values */
 
-  char *default_exportname;
-
   uint64_t exportsize;
+  uint32_t minimum_block_size;
+  uint32_t preferred_block_size;
+  uint32_t maximum_block_size;
   int can_write;
   int can_flush;
   int is_rotational;
@@ -217,39 +236,29 @@ struct handle {
   int can_cache;
 };
 
-static inline void
-reset_handle (struct handle *h)
-{
-  h->handle = NULL;
-  h->state = 0;
-  free (h->default_exportname);
-  h->default_exportname = NULL;
-  h->exportsize = -1;
-  h->can_write = -1;
-  h->can_flush = -1;
-  h->is_rotational = -1;
-  h->can_trim = -1;
-  h->can_zero = -1;
-  h->can_fast_zero = -1;
-  h->can_fua = -1;
-  h->can_multi_conn = -1;
-  h->can_extents = -1;
-  h->can_cache = -1;
-}
+typedef enum {
+  STATUS_DEAD,         /* Connection is closed */
+  STATUS_CLIENT_DONE,  /* Client has sent NBD_CMD_DISC */
+  STATUS_SHUTDOWN,     /* Server wants soft shutdown */
+  STATUS_ACTIVE,       /* Client can make requests */
+} conn_status;
 
 struct connection {
-  pthread_mutex_t request_lock;
-  pthread_mutex_t read_lock;
-  pthread_mutex_t write_lock;
-  pthread_mutex_t status_lock;
+  /* Listed in precedence order: do not grab earlier locks in this list
+   * while holding a later lock.
+   */
+  pthread_mutex_t request_lock; /* Forces serialization of requests */
+  pthread_mutex_t read_lock; /* Read entire client payload off wire */
+  pthread_mutex_t write_lock; /* Protect sockout, write response to wire */
+  pthread_mutex_t status_lock; /* Track current status of client */
 
-  int status; /* 1 for more I/O with client, 0 for shutdown, -1 on error */
+  conn_status status;
   int status_pipe[2]; /* track status changes via poll when nworkers > 1 */
   void *crypto_session;
   int nworkers;
 
-  struct handle *handles;       /* One per plugin and filter. */
-  size_t nr_handles;
+  struct context *top_context;  /* The context tied to 'top'. */
+  char **default_exportname;    /* One per plugin and filter. */
 
   uint32_t cflags;
   uint16_t eflags;
@@ -258,30 +267,27 @@ struct connection {
   bool structured_replies;
   bool meta_context_base_allocation;
 
+  string_vector interns;
   char *exportname_from_set_meta_context;
-  char *exportname;
+  const char *exportname;
 
   int sockin, sockout;
+  /* If nworkers > 1, only call this while read_lock is held */
   connection_recv_function recv;
+  /* If nworkers > 1, only call these while write_lock is held */
   connection_send_function send;
   connection_close_function close;
 };
 
-static inline struct handle *
-get_handle (struct connection *conn, int i)
-{
-  return &conn->handles[i];
-}
-
 extern void handle_single_connection (int sockin, int sockout);
-extern int connection_get_status (void);
-extern int connection_set_status (int value);
+extern conn_status connection_get_status (void);
+extern bool connection_set_status (conn_status value);
 
 /* protocol-handshake.c */
 extern int protocol_handshake (void);
 extern int protocol_common_open (uint64_t *exportsize, uint16_t *flags,
                                  const char *exportname)
-  __attribute__((__nonnull__ (1, 2, 3)));
+  __attribute__ ((__nonnull__ (1, 2, 3)));
 
 /* protocol-handshake-oldstyle.c */
 extern int protocol_handshake_oldstyle (void);
@@ -290,7 +296,7 @@ extern int protocol_handshake_oldstyle (void);
 extern int protocol_handshake_newstyle (void);
 
 /* protocol.c */
-extern int protocol_recv_request_send_reply (void);
+extern bool protocol_recv_request_send_reply (void);
 
 /* The context ID of base:allocation.  As far as I can tell it doesn't
  * matter what this is as long as nbdkit always returns the same
@@ -298,6 +304,9 @@ extern int protocol_recv_request_send_reply (void);
  */
 #define base_allocation_id 1
 
+/* public.c */
+extern void free_interns (void);
+
 /* crypto.c */
 #define root_tls_certificates_dir sysconfdir "/pki/" PACKAGE_NAME
 extern void crypto_init (bool tls_set_on_cli);
@@ -314,16 +323,16 @@ extern void log_verror (const char *fs, va_list args);
 
 /* log-*.c */
 extern void log_stderr_verror (const char *fs, va_list args)
-  __attribute__((__format__ (printf, 1, 0)));
+  ATTRIBUTE_FORMAT_PRINTF (1, 0);
 extern void log_syslog_verror (const char *fs, va_list args)
-  __attribute__((__format__ (printf, 1, 0)));
+  ATTRIBUTE_FORMAT_PRINTF (1, 0);
 
 /* vfprintf.c */
 #if !HAVE_VFPRINTF_PERCENT_M
 #include <stdio.h>
 #define vfprintf replace_vfprintf
 extern int replace_vfprintf (FILE *f, const char *fmt, va_list args)
-  __attribute__((__format__ (printf, 2, 0)));
+  __attribute__ ((__format__ (printf, 2, 0)));
 #endif
 
 /* backend.c */
@@ -364,139 +373,153 @@ struct backend {
   const char *(*magic_config_key) (struct backend *);
   void (*get_ready) (struct backend *);
   void (*after_fork) (struct backend *);
+  void (*cleanup) (struct backend *);
+
   int (*preconnect) (struct backend *, int readonly);
-  int (*list_exports) (struct backend *, int readonly, int default_only,
+  int (*list_exports) (struct backend *, int readonly, int is_tls,
                        struct nbdkit_exports *exports);
-  void *(*open) (struct backend *, int readonly, const char *exportname,
+  const char *(*default_export) (struct backend *, int readonly, int is_tls);
+  void *(*open) (struct context *, int readonly, const char *exportname,
                  int is_tls);
-  int (*prepare) (struct backend *, void *handle, int readonly);
-  int (*finalize) (struct backend *, void *handle);
-  void (*close) (struct backend *, void *handle);
+  int (*prepare) (struct context *, int readonly);
+  int (*finalize) (struct context *);
+  void (*close) (struct context *);
 
-  int64_t (*get_size) (struct backend *, void *handle);
-  int (*can_write) (struct backend *, void *handle);
-  int (*can_flush) (struct backend *, void *handle);
-  int (*is_rotational) (struct backend *, void *handle);
-  int (*can_trim) (struct backend *, void *handle);
-  int (*can_zero) (struct backend *, void *handle);
-  int (*can_fast_zero) (struct backend *, void *handle);
-  int (*can_extents) (struct backend *, void *handle);
-  int (*can_fua) (struct backend *, void *handle);
-  int (*can_multi_conn) (struct backend *, void *handle);
-  int (*can_cache) (struct backend *, void *handle);
+  const char *(*export_description) (struct context *);
+  int64_t (*get_size) (struct context *);
+  int (*block_size) (struct context *,
+                     uint32_t *minimum, uint32_t *preferred, uint32_t *maximum);
+  int (*can_write) (struct context *);
+  int (*can_flush) (struct context *);
+  int (*is_rotational) (struct context *);
+  int (*can_trim) (struct context *);
+  int (*can_zero) (struct context *);
+  int (*can_fast_zero) (struct context *);
+  int (*can_extents) (struct context *);
+  int (*can_fua) (struct context *);
+  int (*can_multi_conn) (struct context *);
+  int (*can_cache) (struct context *);
 
-  int (*pread) (struct backend *, void *handle,
+  int (*pread) (struct context *,
                 void *buf, uint32_t count, uint64_t offset,
                 uint32_t flags, int *err);
-  int (*pwrite) (struct backend *, void *handle,
+  int (*pwrite) (struct context *,
                  const void *buf, uint32_t count, uint64_t offset,
                  uint32_t flags, int *err);
-  int (*flush) (struct backend *, void *handle, uint32_t flags, int *err);
-  int (*trim) (struct backend *, void *handle,
+  int (*flush) (struct context *, uint32_t flags, int *err);
+  int (*trim) (struct context *,
                uint32_t count, uint64_t offset, uint32_t flags, int *err);
-  int (*zero) (struct backend *, void *handle,
+  int (*zero) (struct context *,
                uint32_t count, uint64_t offset, uint32_t flags, int *err);
-  int (*extents) (struct backend *, void *handle,
+  int (*extents) (struct context *,
                   uint32_t count, uint64_t offset, uint32_t flags,
                   struct nbdkit_extents *extents, int *err);
-  int (*cache) (struct backend *, void *handle,
+  int (*cache) (struct context *,
                 uint32_t count, uint64_t offset, uint32_t flags, int *err);
 };
 
 extern void backend_init (struct backend *b, struct backend *next, size_t index,
                           const char *filename, void *dl, const char *type)
-  __attribute__((__nonnull__ (1, 4, 5, 6)));
+  __attribute__ ((__nonnull__ (1, 4, 5, 6)));
 extern void backend_load (struct backend *b, const char *name,
                           void (*load) (void))
-  __attribute__((__nonnull__ (1 /* not 2 */)));
+  __attribute__ ((__nonnull__ (1 /* not 2 */)));
 extern void backend_unload (struct backend *b, void (*unload) (void))
-  __attribute__((__nonnull__ (1)));
+  __attribute__ ((__nonnull__ (1)));
 
 extern int backend_list_exports (struct backend *b, int readonly,
-                                 int default_only,
                                  struct nbdkit_exports *exports)
-  __attribute__((__nonnull__ (1, 4)));
+  __attribute__ ((__nonnull__ (1, 3)));
+extern const char *backend_default_export (struct backend *b, int readonly)
+  __attribute__ ((__nonnull__ (1)));
 /* exportname is only valid for this call and almost certainly will be
  * freed on return of this function, so backends must save the
  * exportname if they need to refer to it later.
  */
-extern int backend_open (struct backend *b,
-                         int readonly, const char *exportname)
-  __attribute__((__nonnull__ (1, 3)));
-extern int backend_prepare (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_finalize (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern void backend_close (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern bool backend_valid_range (struct backend *b,
+extern struct context *backend_open (struct backend *b,
+                                     int readonly, const char *exportname,
+                                     int shared)
+  __attribute__ ((__nonnull__ (1, 3)));
+extern int backend_prepare (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_finalize (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern void backend_close (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern bool backend_valid_range (struct context *c,
                                  uint64_t offset, uint32_t count)
-  __attribute__((__nonnull__ (1)));
+  __attribute__ ((__nonnull__ (1)));
 
-extern int backend_reopen (struct backend *b,
-                           int readonly, const char *exportname)
-  __attribute__((__nonnull__ (1, 3)));
-extern int64_t backend_get_size (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_write (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_flush (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_is_rotational (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_trim (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_zero (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_fast_zero (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_extents (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_fua (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_multi_conn (struct backend *b)
-  __attribute__((__nonnull__ (1)));
-extern int backend_can_cache (struct backend *b)
-  __attribute__((__nonnull__ (1)));
+extern const char *backend_export_description (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int64_t backend_get_size (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_block_size (struct context *c,
+                               uint32_t *minimum, uint32_t *preferred,
+                               uint32_t *maximum)
+  __attribute__ ((__nonnull__ (1, 2, 3, 4)));
+extern int backend_can_write (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_can_flush (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_is_rotational (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_can_trim (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_can_zero (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_can_fast_zero (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_can_extents (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_can_fua (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_can_multi_conn (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
+extern int backend_can_cache (struct context *c)
+  __attribute__ ((__nonnull__ (1)));
 
-extern int backend_pread (struct backend *b,
+extern int backend_pread (struct context *c,
                           void *buf, uint32_t count, uint64_t offset,
                           uint32_t flags, int *err)
-  __attribute__((__nonnull__ (1, 2, 6)));
-extern int backend_pwrite (struct backend *b,
+  __attribute__ ((__nonnull__ (1, 2, 6)));
+extern int backend_pwrite (struct context *c,
                            const void *buf, uint32_t count, uint64_t offset,
                            uint32_t flags, int *err)
-  __attribute__((__nonnull__ (1, 2, 6)));
-extern int backend_flush (struct backend *b,
+  __attribute__ ((__nonnull__ (1, 2, 6)));
+extern int backend_flush (struct context *c,
                           uint32_t flags, int *err)
-  __attribute__((__nonnull__ (1, 3)));
-extern int backend_trim (struct backend *b,
+  __attribute__ ((__nonnull__ (1, 3)));
+extern int backend_trim (struct context *c,
                          uint32_t count, uint64_t offset, uint32_t flags,
                          int *err)
-  __attribute__((__nonnull__ (1, 5)));
-extern int backend_zero (struct backend *b,
+  __attribute__ ((__nonnull__ (1, 5)));
+extern int backend_zero (struct context *c,
                          uint32_t count, uint64_t offset, uint32_t flags,
                          int *err)
-  __attribute__((__nonnull__ (1, 5)));
-extern int backend_extents (struct backend *b,
+  __attribute__ ((__nonnull__ (1, 5)));
+extern int backend_extents (struct context *c,
                             uint32_t count, uint64_t offset, uint32_t flags,
                             struct nbdkit_extents *extents, int *err)
-  __attribute__((__nonnull__ (1, 5, 6)));
-extern int backend_cache (struct backend *b,
+  __attribute__ ((__nonnull__ (1, 5, 6)));
+extern int backend_cache (struct context *c,
                           uint32_t count, uint64_t offset,
                           uint32_t flags, int *err)
-  __attribute__((__nonnull__ (1, 5)));
+  __attribute__ ((__nonnull__ (1, 5)));
 
 /* plugins.c */
+typedef struct nbdkit_plugin *(*plugin_init_function) (void);
 extern struct backend *plugin_register (size_t index, const char *filename,
-                                        void *dl, struct nbdkit_plugin *(*plugin_init) (void))
-  __attribute__((__nonnull__ (2, 3, 4)));
+                                        void *dl,
+                                        plugin_init_function plugin_init)
+  __attribute__ ((__nonnull__ (2, 3, 4)));
 
 /* filters.c */
+typedef struct nbdkit_filter *(*filter_init_function) (void);
 extern struct backend *filter_register (struct backend *next, size_t index,
                                         const char *filename, void *dl,
-                                        struct nbdkit_filter *(*filter_init) (void))
-  __attribute__((__nonnull__ (1, 3, 4, 5)));
+                                        filter_init_function filter_init)
+  __attribute__ ((__nonnull__ (1, 3, 4, 5)));
 
 /* locks.c */
 extern unsigned thread_model;
@@ -510,18 +533,18 @@ extern void lock_unload (void);
 extern void unlock_unload (void);
 
 /* sockets.c */
-DEFINE_VECTOR_TYPE(sockets, int);
-extern void bind_unix_socket (sockets *) __attribute__((__nonnull__ (1)));
-extern void bind_tcpip_socket (sockets *) __attribute__((__nonnull__ (1)));
-extern void bind_vsock (sockets *) __attribute__((__nonnull__ (1)));
+DEFINE_VECTOR_TYPE (sockets, int);
+extern void bind_unix_socket (sockets *) __attribute__ ((__nonnull__ (1)));
+extern void bind_tcpip_socket (sockets *) __attribute__ ((__nonnull__ (1)));
+extern void bind_vsock (sockets *) __attribute__ ((__nonnull__ (1)));
 extern void accept_incoming_connections (const sockets *socks)
-  __attribute__((__nonnull__ (1)));
+  __attribute__ ((__nonnull__ (1)));
 
 /* threadlocal.c */
 extern void threadlocal_init (void);
 extern void threadlocal_new_server_thread (void);
 extern void threadlocal_set_name (const char *name)
-  __attribute__((__nonnull__ (1)));
+  __attribute__ ((__nonnull__ (1)));
 extern const char *threadlocal_get_name (void);
 extern void threadlocal_set_instance_num (size_t instance_num);
 extern size_t threadlocal_get_instance_num (void);
@@ -530,6 +553,14 @@ extern int threadlocal_get_error (void);
 extern void *threadlocal_buffer (size_t size);
 extern void threadlocal_set_conn (struct connection *conn);
 extern struct connection *threadlocal_get_conn (void);
+extern struct context *threadlocal_get_context (void);
+
+extern struct context *threadlocal_push_context (struct context *ctx);
+extern void threadlocal_pop_context (struct context **ctx);
+#define CLEANUP_CONTEXT_POP __attribute__ ((cleanup (threadlocal_pop_context)))
+#define PUSH_CONTEXT_FOR_SCOPE(ctx)                                     \
+  CLEANUP_CONTEXT_POP CLANG_UNUSED_VARIABLE_WORKAROUND                  \
+  struct context *NBDKIT_UNIQUE_NAME (_ctx) = threadlocal_push_context (ctx)
 
 /* Macro which sets local variable struct connection *conn from
  * thread-local storage, asserting that it is non-NULL.  If you want
@@ -540,4 +571,8 @@ extern struct connection *threadlocal_get_conn (void);
   struct connection *conn = threadlocal_get_conn ();    \
   assert (conn != NULL)
 
+/* exports.c */
+extern int exports_resolve_default (struct nbdkit_exports *exports,
+                                    struct backend *b, int readonly);
+
 #endif /* NBDKIT_INTERNAL_H */
