@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2020 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -79,8 +79,10 @@
 #endif
 
 static char *make_random_fifo (void);
-static struct backend *open_plugin_so (size_t i, const char *filename, int short_name);
-static struct backend *open_filter_so (struct backend *next, size_t i, const char *filename, int short_name);
+static struct backend *open_plugin_so (size_t i, const char *filename,
+                                       int short_name);
+static struct backend *open_filter_so (struct backend *next, size_t i,
+                                       const char *filename, int short_name);
 static void start_serving (void);
 static void write_pidfile (void);
 static bool is_config_key (const char *key, size_t len);
@@ -372,7 +374,7 @@ main (int argc, char *argv[])
       break;
 
     case VSOCK_OPTION:
-#if defined(AF_VSOCK) && defined(VMADDR_CID_ANY)
+#if defined (AF_VSOCK) && defined (VMADDR_CID_ANY)
       vsock = true;
       break;
 #else
@@ -572,9 +574,11 @@ main (int argc, char *argv[])
   /* Implement --exit-with-parent early in case plugin initialization
    * takes a long time and the parent exits during that time.
    */
-  if (set_exit_with_parent () == -1) {
-    perror ("nbdkit: --exit-with-parent");
-    exit (EXIT_FAILURE);
+  if (exit_with_parent) {
+    if (set_exit_with_parent () == -1) {
+      perror ("nbdkit: --exit-with-parent");
+      exit (EXIT_FAILURE);
+    }
   }
 
   /* If the user has mixed up -p/--run/-s/-U/--vsock options, then
@@ -1092,7 +1096,7 @@ error_if_stdio_closed (void)
 static void
 switch_stdio (void)
 {
-#if defined(F_DUPFD_CLOEXEC) || defined(F_DUPFD)
+#if defined (F_DUPFD_CLOEXEC) || defined (F_DUPFD)
   fflush (stdin);
   fflush (NULL);
   if (listen_stdin || run) {
