@@ -1,5 +1,5 @@
 /* nbdkit
- * Copyright (C) 2013-2022 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -711,7 +711,7 @@ nbdkit_nanosleep (unsigned sec, unsigned nsec)
   /* Block all signals to this thread during the poll, so we don't
    * have to worry about EINTR
    */
-  if (sigfillset(&all))
+  if (sigfillset (&all))
     abort ();
   switch (ppoll (fds, ARRAY_SIZE (fds), &ts, &all)) {
   case -1:
@@ -727,7 +727,8 @@ nbdkit_nanosleep (unsigned sec, unsigned nsec)
    */
   bool has_quit = quit;
   assert (has_quit ||
-          (conn && conn->nworkers > 0 && connection_get_status () < 1) ||
+          (conn && conn->nworkers > 0 &&
+           connection_get_status () < STATUS_SHUTDOWN) ||
           (conn && (fds[2].revents & (POLLRDHUP | POLLHUP | POLLERR |
                                       POLLNVAL))));
   if (has_quit)
@@ -825,8 +826,8 @@ nbdkit_peer_name (struct sockaddr *addr, socklen_t *addrlen)
   return 0;
 }
 
-#if defined(SO_PEERCRED) && \
-  (defined(HAVE_STRUCT_UCRED_UID) || defined(HAVE_STRUCT_SOCKPEERCRED_UID))
+#if defined (SO_PEERCRED) && \
+  (defined (HAVE_STRUCT_UCRED_UID) || defined (HAVE_STRUCT_SOCKPEERCRED_UID))
 
 #define GET_PEERCRED_DEFINED 1
 
@@ -1012,8 +1013,7 @@ free_interns (void)
   struct connection *conn = threadlocal_get_conn ();
   string_vector *list = conn ? &conn->interns : &global_interns;
 
-  string_vector_iter (list, (void *) free);
-  string_vector_reset (list);
+  string_vector_empty (list);
 }
 
 static const char *
@@ -1096,3 +1096,18 @@ nbdkit_printf_intern (const char *fmt, ...)
   va_end (ap);
   return ret;
 }
+
+NBDKIT_DLL_PUBLIC void
+nbdkit_disconnect (int force)
+{
+  struct connection *conn = threadlocal_get_conn ();
+
+  if (!conn) {
+    debug ("no connection in this thread, ignoring disconnect request");
+    return;
+  }
+  if (connection_set_status (force ? STATUS_DEAD : STATUS_SHUTDOWN)) {
+    ACQUIRE_LOCK_FOR_CURRENT_SCOPE (&conn->write_lock);
+    conn->close (SHUT_WR);
+  }
+}
