@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2018-2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -34,16 +34,12 @@ source ./functions.sh
 set -e
 set -x
 
+requires_run
 requires hexdump --version
 requires $PYTHON --version
 requires_nbdcopy
 requires_plugin python
-
-# Python has proven very difficult to valgrind, therefore it is disabled.
-if [ "$NBDKIT_VALGRIND" = "1" ]; then
-    echo "$0: skipping Python test under valgrind."
-    exit 77
-fi
+skip_if_valgrind "because Python code leaks memory"
 
 # There is a fake boto3 module in test-S3/ which we use as a test
 # harness for the plugin.
