@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -37,17 +37,13 @@ source ./functions.sh
 set -e
 set -x
 
-if test "$SRCDIR" = ""; then
-    echo "$0: \$SRCDIR is not set"
-    exit 1
-fi
-
-script=$SRCDIR/test-cc-cpp.cpp
+script=$abs_top_srcdir/tests/test-cc-cpp.cpp
 if test ! -f "$script"; then
     echo "$0: could not locate test-cc-cpp.cpp"
     exit 1
 fi
 
+requires_run
 requires_plugin cc
 requires $CXX --version
 requires_nbdsh_uri
@@ -59,6 +55,6 @@ rm -f $out
 
 nbdkit -U - cc $script \
        CC="$CXX" \
-       EXTRA_CFLAGS="-I$SRCDIR/../include" \
+       EXTRA_CFLAGS="-I$abs_top_srcdir/include" \
        --run 'nbdinfo --size $uri' > $out
 test "$(cat $out)" -eq $((100 * 1024 * 1024))
