@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -37,17 +37,13 @@ source ./functions.sh
 set -e
 set -x
 
-if test "$SRCDIR" = ""; then
-    echo "$0: \$SRCDIR is not set"
-    exit 1
-fi
-
-script=$SRCDIR/test_ocaml_plugin.ml
+script=$abs_top_srcdir/tests/test_ocaml_plugin.ml
 if test ! -f "$script"; then
     echo "$0: could not locate test_ocaml_plugin.ml"
     exit 1
 fi
 
+requires_run
 requires_plugin cc
 requires $OCAMLOPT -version
 requires_nbdsh_uri
@@ -60,7 +56,7 @@ out=test-cc-ocaml.out
 cleanup_fn rm -f $out
 rm -f $out
 
-nbdkit -U - cc $script a=1 b=2 c=3 d=4 \
-       CC="$OCAMLOPT" CFLAGS="-output-obj -runtime-variant _pic -I $SRCDIR/../plugins/ocaml unix.cmxa NBDKit.cmx -cclib -lnbdkitocaml" \
+nbdkit -v -U - cc $script a=1 b=2 c=3 d=4 \
+       CC="$OCAMLOPT" CFLAGS="-output-obj -runtime-variant _pic -I $abs_top_srcdir/plugins/ocaml $OCAML_PLUGIN_LIBRARIES NBDKit.cmx -cclib -L../plugins/ocaml/.libs -cclib -lnbdkitocaml" \
        --run 'nbdinfo --size $uri' > $out
 test "$(cat $out)" -eq $((512 * 2048))
