@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2018 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -37,7 +37,7 @@ set -x
 
 requires_plugin example2
 requires_run
-requires nbdinfo --version
+requires_nbdinfo
 requires test -f disk
 
 export LANG=C
