@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2018-2022 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -34,6 +34,8 @@ source ./functions.sh
 set -e
 set -x
 
+requires_run
+requires_nbdsh_uri
 requires $STAT --version
 
 sock2=$(mktemp -u /tmp/nbdkit-test-sock.XXXXXX)
