@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2017-2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -32,10 +32,10 @@
 
 source ./functions.sh
 
-# Check file-data was created by Makefile and qemu-io exists.
+requires_run
 requires test -f file-data
 requires qemu-io --version
-requires timeout --version
+requires timeout 60s true
 
 nbdkit --dump-plugin nbd | grep -q ^thread_model=parallel ||
     { echo "nbdkit lacks support for parallel requests"; exit 77; }
