@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2018-2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -34,6 +34,8 @@ source ./functions.sh
 set -e
 set -x
 
+requires_run
+requires_plugin sh
 requires qemu-io --version
 
 out="test-sh-errors.out"
@@ -66,7 +68,7 @@ do_test ()
 }
 
 # All caps, no trailing whitespace, ENOMEM
-do_test 'Cannot allocate memory' <<'EOF'
+do_test '(Cannot allocate|Out of) memory' <<'EOF'
 case "$1" in
     get_size) echo 64K ;;
     pread) printf ENOMEM >&2; exit 1 ;;
@@ -102,7 +104,7 @@ esac
 EOF
 
 # Any errno that can't be sent over NBD is flattened to EIO
-do_test 'Input/output error' <<'EOF'
+do_test '(Input/output|I/O) error' <<'EOF'
 case "$1" in
     get_size) echo 1M ;;
     pread) echo EPIPE >&2; exit 1 ;;
@@ -111,7 +113,7 @@ esac
 EOF
 
 # EINVALID is not a real errno name, and should not be confused with EINVAL
-do_test 'Input/output error' <<'EOF'
+do_test '(Input/output| I/O) error' <<'EOF'
 case "$1" in
     get_size) echo 1M ;;
     pread) echo EINVALID >&2; exit 1 ;;
