@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2018-2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -39,22 +39,22 @@ set -x
 requires_run
 requires_nbdsh_uri
 requires nbdsh --base-allocation
-requires truncate --help
-requires stat --help
+requires $TRUNCATE --help
+requires $STAT --help
 
 files="test-split-extents.1 test-split-extents.2"
 rm -f $files
 cleanup_fn rm -f $files
 
 # Create two files, each half data and half sparse
-truncate --size=512k test-split-extents.1
-if test "$(stat -c %b test-split-extents.1)" != 0; then
+$TRUNCATE -s 512k test-split-extents.1
+if test "$($STAT -c %b test-split-extents.1)" != 0; then
     echo "$0: unable to create sparse file, skipping this test"
     exit 77
 fi
 printf %$((512*1024))d 1 >> test-split-extents.1
 printf %$((512*1024))d 1 >> test-split-extents.2
-truncate --size=1M test-split-extents.2
+$TRUNCATE -s 1M test-split-extents.2
 
 # Test the split plugin
 nbdkit -v -U - split test-split-extents.1 test-split-extents.2 \
