@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2017-2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -36,8 +36,8 @@ source ./functions.sh
 set -e
 set -x
 
+requires_run
 requires test -f disk
-requires guestfish --version
 requires tar --version
 requires qemu-img --version
 requires qemu-img info --output=json /dev/null
