@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # nbdkit
-# Copyright (C) 2018-2020 Red Hat Inc.
+# Copyright Red Hat
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -40,12 +40,8 @@ source ./functions.sh
 set -e
 set -x
 
-# Setting LD_LIBRARY_PATH breaks valgrind.
-if [ "x$NBDKIT_VALGRIND" = "x1" ]; then
-    echo "$0: skipped under valgrind"
-    exit 77
-fi
-
+skip_if_valgrind "because setting LD_LIBRARY_PATH breaks valgrind"
+requires_run
 requires_nbdinfo
 
 f=test-vddk-password-fd.file
