@@ -1,5 +1,5 @@
 (* nbdkit
- * Copyright (C) 2013-2021 Red Hat Inc.
+ * Copyright Red Hat
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -30,6 +30,11 @@
  * SUCH DAMAGE.
  *)
 
+(* Print something during module initialization, useful for debugging
+ * obscure OCaml startup issues.
+ *)
+let () = Printf.eprintf "test_ocaml_plugin.ml: module initializing\n%!"
+
 let sector_size = 512
 let nr_sectors = 2048
 
@@ -52,6 +57,16 @@ let () =
      *)
     assert (isdir (NBDKit.realpath test_dir))
 
+(* Test [NBDKit.version ()] returns a sensible looking string. *)
+let () =
+  let ver = NBDKit.version () in
+  assert (String.length ver > 2);
+  assert (String.sub ver 0 2 = "1.")
+
+(* Test [NBDKit.api_version ()]. *)
+let () =
+  assert (NBDKit.api_version () = 2)
+
 let load () =
   NBDKit.debug "test ocaml plugin loaded"
 
